@@ -69,7 +69,7 @@ void QixDemo(int rop, char *txt);
 
 void convxbm(int width, int height, unsigned char *bits, unsigned char *conv_bits);
 
-void main(void)
+int main(void)
 {
     vga_modeinfo *minfo;
 
