@@ -382,6 +382,12 @@ extern "C"
     extern int vga_ext_set(unsigned what,...);
     extern int vga_accel(unsigned operation,...);
 
+    extern int vga_initcursor(int);
+    extern void vga_showcursor(int);
+    extern void vga_setcursorposition(int, int);
+    extern void vga_selectcursor(int);
+    extern void vga_setcursorimage(int, int, int, int, unsigned char *);
+
     extern int vga_addtiming(int pixelClock,
        			      int HDisplay,		
                               int HSyncStart,
