@@ -37,9 +37,25 @@ static int svgalib_helper_ioctl( struct inode *inode, struct file *filp,
 
     io_t iov;
     int minor = MINOR(inode->i_rdev);
+    
+    io_string_t iostr;
+    int i;
+    unsigned char *outb_str;
 
     if(_IOC_TYPE(cmd)!=SVGALIB_HELPER_IOC_MAGIC) return -EINVAL;
     switch(_IOC_NR(cmd)) {
+        case _IOC_NR(SVGALIB_HELPER_IOCSREPOUTB):
+  	   copy_from_user(&iostr,(char *)arg,sizeof(iostr));
+	   if ( iostr.length > 768 )
+	     return -EINVAL;
+	   if ( (outb_str = kmalloc(iostr.length, GFP_KERNEL )) == NULL )
+	     return -ENOMEM;
+	   copy_from_user(outb_str,iostr.string,iostr.length);
+	   check_io_range(iostr.port,minor);
+	   for ( i = 0; i < iostr.length; i++ )
+	     outb ( outb_str[i], iostr.port );
+	   kfree ( outb_str );
+        break;
         case _IOC_NR(SVGALIB_HELPER_IOCSOUTB):
            copy_from_user(&iov,(char *)arg,sizeof(iov));
            check_io_range(iov.port,minor);
