@@ -60,6 +60,7 @@ typedef struct {
     int (*linear) (int op, int param);
     AccelSpecs *accelspecs;
     Emulation *emul;
+    int (*cursor)(int cmd, int p1, int p2, int p3, int p4, void *p5);
 } DriverSpecs;
 
 extern DriverSpecs __svgalib_vga_driverspecs;
@@ -108,6 +109,10 @@ enum {
     LINEAR_ENABLE, LINEAR_DISABLE
 };
 
+enum { CURSOR_INIT, CURSOR_HIDE, CURSOR_SHOW, CURSOR_POSITION, 
+       CURSOR_SELECT, CURSOR_IMAGE 
+};
+
 typedef struct {
 /* refresh ranges in Hz */
     unsigned min;
