@@ -44,11 +44,18 @@ typedef struct {
     unsigned char InterlaceControl;
     unsigned char ExtStartAddr;
     unsigned char ExtStartAddrHi;    
-    unsigned int  LMI_FIFO_Watermark;
+    unsigned char CursorControl;
+    unsigned char CursorBaseAddrLo;
+    unsigned char CursorBaseAddrHi;
+    unsigned char CursorXLo;
+    unsigned char CursorXHi;
+    unsigned char CursorYLo;
+    unsigned char CursorYHi;
+    unsigned int  LMI_FIFO_Watermark; 
 } vgaI740Rec, *vgaI740Ptr;
 
 #define I740REG_SAVE(i) (VGA_TOTAL_REGS+i)
-#define I740_TOTAL_REGS (VGA_TOTAL_REGS + 26)
+#define I740_TOTAL_REGS (VGA_TOTAL_REGS + sizeof(vgaI740Rec))
 
 static int i740_init(int, int, int);
 static void i740_unlock(void);
@@ -127,19 +134,14 @@ static int i740_saveregs(unsigned char regs[])
     i740_unlock();		
     save = (vgaI740Ptr)(regs+VGA_TOTAL_REGS);
 
-    /*
-     * The port I/O code necessary to read in the extended registers 
-     * into the fields of the vgaI740Rec structure goes here.
-     */
-
-    i740_outxr(IO_CTNL, save->IOControl);
-    i740_outxr(ADDRESS_MAPPING, save->AddressMapping);
-    i740_outxr(BITBLT_CNTL, save->BitBLTControl);
-    i740_outxr(VCLK2_VCO_M, save->VideoClk2_M);
-    i740_outxr(VCLK2_VCO_N, save->VideoClk2_N);
-    i740_outxr(VCLK2_VCO_MN_MSBS, save->VideoClk2_MN_MSBs);
-    i740_outxr(VCLK2_VCO_DIV_SEL, save->VideoClk2_DivisorSel);
-    i740_outxr(PLL_CNTL, save->PLLControl);
+    save->IOControl=i740_inxr(IO_CTNL);
+    save->AddressMapping=i740_inxr(ADDRESS_MAPPING);
+    save->BitBLTControl=i740_inxr(BITBLT_CNTL);
+    save->VideoClk2_M=i740_inxr(VCLK2_VCO_M);
+    save->VideoClk2_N=i740_inxr(VCLK2_VCO_N);
+    save->VideoClk2_MN_MSBs=i740_inxr(VCLK2_VCO_MN_MSBS);
+    save->VideoClk2_DivisorSel=i740_inxr(VCLK2_VCO_DIV_SEL);
+    save->PLLControl=i740_inxr(PLL_CNTL);
 
     save->ExtVertTotal = __svgalib_incrtc(EXT_VERT_TOTAL);
     save->ExtVertDispEnd = __svgalib_incrtc(EXT_VERT_DISPLAY);
@@ -152,10 +154,18 @@ static int i740_saveregs(unsigned char regs[])
     save->ExtStartAddr = __svgalib_incrtc(EXT_START_ADDR);
     save->ExtStartAddrHi = __svgalib_incrtc(EXT_START_ADDR_HI);
 
-    i740_outxr(PIXPIPE_CONFIG_0, save->PixelPipeCfg0);
-    i740_outxr(PIXPIPE_CONFIG_1, save->PixelPipeCfg1);
-    i740_outxr(PIXPIPE_CONFIG_2, save->PixelPipeCfg2);
-    i740_outxr(DISPLAY_CNTL, save->DisplayControl);
+    save->PixelPipeCfg0=i740_inxr(PIXPIPE_CONFIG_0);
+    save->PixelPipeCfg1=i740_inxr(PIXPIPE_CONFIG_1);
+    save->PixelPipeCfg2=i740_inxr(PIXPIPE_CONFIG_2);
+    save->DisplayControl=i740_inxr(DISPLAY_CNTL);
+
+    save->CursorControl=i740_inxr(CURSOR_CONTROL);
+    save->CursorBaseAddrHi=i740_inxr(CURSOR_BASEADDR_HI);
+    save->CursorBaseAddrLo=i740_inxr(CURSOR_BASEADDR_LO);
+    save->CursorXHi=i740_inxr(CURSOR_X_HI);
+    save->CursorXLo=i740_inxr(CURSOR_X_LO);
+    save->CursorYHi=i740_inxr(CURSOR_Y_HI);
+    save->CursorYLo=i740_inxr(CURSOR_Y_LO);
 
     save->LMI_FIFO_Watermark = INREG(FWATER_BLC);
 
@@ -183,8 +193,8 @@ static void i740_setregs(const unsigned char regs[], int mode)
     i740_outxr(VCLK2_VCO_DIV_SEL, restore->VideoClk2_DivisorSel);
 
     temp = i740_inxr(PIXPIPE_CONFIG_0);
-    temp &= 0x7F; /* Save all but the 8 bit dac mode bit */
-    temp |= (restore->PixelPipeCfg0 & DAC_8_BIT);
+    temp &= ~(DAC_8_BIT | HW_CURSOR_ENABLE); 
+    temp |= (restore->PixelPipeCfg0 & (DAC_8_BIT | HW_CURSOR_ENABLE));
     i740_outxr(PIXPIPE_CONFIG_0, temp);
 
     __svgalib_outcrtc(EXT_VERT_TOTAL, restore->ExtVertTotal);
@@ -204,7 +214,15 @@ static void i740_setregs(const unsigned char regs[], int mode)
     __svgalib_outcrtc(EXT_START_ADDR, restore->ExtStartAddr);
     __svgalib_outcrtc(EXT_START_ADDR_HI, restore->ExtStartAddrHi);
 
-    temp=__svgalib_inCR(INTERLACE_CNTL); 
+    i740_outxr(CURSOR_CONTROL,restore->CursorControl);
+    i740_outxr(CURSOR_BASEADDR_HI,restore->CursorBaseAddrHi);
+    i740_outxr(CURSOR_BASEADDR_LO,restore->CursorBaseAddrLo);
+    i740_outxr(CURSOR_X_HI,restore->CursorXHi);
+    i740_outxr(CURSOR_X_LO,restore->CursorXLo);
+    i740_outxr(CURSOR_Y_HI,restore->CursorYHi);
+    i740_outxr(CURSOR_Y_LO,restore->CursorYLo);
+
+    temp=__svgalib_incrtc(INTERLACE_CNTL); 
     temp &= ~INTERLACE_ENABLE;
     temp |= restore->InterlaceControl;
     __svgalib_outcrtc(INTERLACE_CNTL, temp);
@@ -537,6 +555,8 @@ static void i740_initializemode(unsigned char *moderegs,
     /* Calculate the FIFO Watermark and Burst Length. */
     new->LMI_FIFO_Watermark = I740CalcFIFO(modetiming->pixelClock/1000,modeinfo->bitsPerPixel);
 
+    new->CursorControl= CURSOR_ORIGIN_DISPLAY | CURSOR_MODE_32_4C_AX;
+
     return ;
 }
 
@@ -614,7 +634,6 @@ static int i740_test(void)
     return 0;
 }
 
-
 /* Set display start address (not for 16 color modes) */
 /* Cirrus supports any address in video memory (up to 2Mb) */
 
@@ -636,7 +655,7 @@ static void i740_setlogicalwidth(int width)
     int offset = width >> 3;
  
     __svgalib_outcrtc(0x13,offset&0xff);
-    __svgalib_outCR(EXT_OFFSET, offset>>8);
+    __svgalib_outcrtc(EXT_OFFSET, offset>>8);
 }
 
 static int i740_linear(int op, int param)
@@ -648,6 +667,73 @@ static int i740_linear(int op, int param)
         else return -1;		/* Unknown function. */
 }
 
+static unsigned char cur_colors[16*6];
+
+static int i740_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
+    int i, j, tmp;
+    unsigned long *b3;
+    unsigned long l1, l2, l3, l4;
+    
+    switch(cmd){
+        case CURSOR_INIT:
+            return 1;
+        case CURSOR_HIDE:
+            i740_outxr(PIXPIPE_CONFIG_0,i740_inxr(PIXPIPE_CONFIG_0)&~HW_CURSOR_ENABLE); 
+            break;
+        case CURSOR_SHOW:
+            i740_outxr(PIXPIPE_CONFIG_0,i740_inxr(PIXPIPE_CONFIG_0)|HW_CURSOR_ENABLE); 
+            break;
+        case CURSOR_POSITION:
+            i740_outxr(CURSOR_X_LO,p1&0xff);
+            i740_outxr(CURSOR_X_HI,(p1&0x700)>>8);
+            i740_outxr(CURSOR_Y_LO,p2&0xff);
+            i740_outxr(CURSOR_Y_HI,(p2&0x700)>>8);
+            break;
+        case CURSOR_SELECT:
+            i=i740_memory*1024-(p1+1)*4096;
+            if(i>4096*1024)i-=4096*1024;
+            tmp=i740_inxr(PIXPIPE_CONFIG_0);
+            i740_outxr(PIXPIPE_CONFIG_0,tmp| EXTENDED_PALETTE);
+            __svgalib_outpal(4,cur_colors[p1*6],cur_colors[p1*6+1],cur_colors[p1*6+2]);
+            __svgalib_outpal(5,cur_colors[p1*6+3],cur_colors[p1*6+4],cur_colors[p1*6+5]);
+            i740_outxr(PIXPIPE_CONFIG_0,tmp);
+            i740_outxr(CURSOR_BASEADDR_LO,i>>8); 
+            i740_outxr(CURSOR_BASEADDR_HI,i>>16); 
+            break;
+        case CURSOR_IMAGE:
+            i=i740_memory*1024-(p1+1)*4096;
+            if(i>4096*1024)i-=4096*1024;
+            b3=(unsigned long *)p5;
+            switch(p2) {
+                case 0:
+                    cur_colors[p1*6]=(p3&0xff0000)>>16;
+                    cur_colors[p1*6+1]=(p3&0xff00)>>8;
+                    cur_colors[p1*6+2]=p3&0xff;
+                    cur_colors[p1*6+3]=(p4&0xff0000)>>16;
+                    cur_colors[p1*6+4]=(p4&0xff00)>>8;
+                    cur_colors[p1*6+5]=p4&0xff;
+                    for(j=0;j<16;j++) {
+                        l1=*(b3+2*j);
+                        l2=~(*(b3+32+2*j));
+                        l3=*(b3+2*j+1);
+                        l4=~(*(b3+32+2*j+1));
+                        /*change endianess */
+                        l1=(l1<<24)|(l1>>24)|((l1>>8)&0xff00)|((l1<<8)&0xff0000);
+                        l2=(l2<<24)|(l2>>24)|((l2>>8)&0xff00)|((l2<<8)&0xff0000);
+                        l3=(l3<<24)|(l3>>24)|((l3>>8)&0xff00)|((l3<<8)&0xff0000);
+                        l4=(l4<<24)|(l4>>24)|((l4>>8)&0xff00)|((l4<<8)&0xff0000);
+                        *(unsigned long *)(LINEAR_MEM_POINTER+i+16*j)=l2;
+                        *(unsigned long *)(LINEAR_MEM_POINTER+i+16*j+4)=l4;
+                        *(unsigned long *)(LINEAR_MEM_POINTER+i+16*j+8)=l1;
+                        *(unsigned long *)(LINEAR_MEM_POINTER+i+16*j+12)=l3;
+                    }
+                break;
+            }
+            break;
+    }
+    return 0;
+}       
+
 static int i740_match_programmable_clock(int clock)
 {
 return clock ;
@@ -691,10 +777,10 @@ DriverSpecs __svgalib_i740_driverspecs =
     i740_linear,
     0,				/* accelspecs, filled in during init. */
     NULL,                       /* Emulation */
+    i740_cursor
 };
 
 /* Initialize chipset (called after detection) */
-
 static int i740_init(int force, int par1, int par2)
 {
     unsigned long buf[64];
