@@ -1,4 +1,4 @@
-
+#define MS3B
 /* Based on:
  * simple driver for serial mouse
  * Andrew Haylett, 14th December 1992
@@ -575,10 +575,16 @@ static int ms_init(void)
     case MOUSE_BUSMOUSE:
     case MOUSE_PS2:
     case MOUSE_GPM:
+    case MOUSE_PNP:
+#ifdef MS3B
+    case MOUSE_MICROSOFT: /* Two buttons only */
+#endif
         /* Any of these _can_ have 3 buttons, but may not */
         mouse_caps.buttons |= MOUSE_MIDDLEBUTTON;
 
+#ifndef MS3B
     case MOUSE_MICROSOFT: /* Two buttons only */
+#endif
         mouse_caps.buttons |= MOUSE_LEFTBUTTON | MOUSE_RIGHTBUTTON;
         mouse_caps.axes |= MOUSE_2DIM;
         break;
@@ -624,6 +630,7 @@ static int get_ms_event(int wait) {
     char event_handled = 0;
     int bytesread;
     int i, wheel;
+    static unsigned int prev=0;
 /*  int but; */
     static int but = 0;		/* static is hack for MouseMan */
     static int mouse_orientation = 0;
@@ -643,17 +650,8 @@ static int get_ms_event(int wait) {
 	fcntl(__svgalib_mouse_fd, F_SETFL, O_RDONLY | O_NDELAY );
 	m_fdmode = 0;
     }
-#ifdef BACKGROUND
-    /* No need to read mouse while background. */
-    if (vga_oktowrite()) {
-#endif
         bytesread = read(__svgalib_mouse_fd,
 		     &buf[nu_bytes], MOUSEBUFFERSIZE - nu_bytes);
-#ifdef BACKGROUND
-       }
-     else
-       bytesread=0;
-#endif
     i = 0;
 
     if (bytesread >= 1)
@@ -714,10 +712,6 @@ static int get_ms_event(int wait) {
 	    /* Wait mode, we'll sleep on reads. */
 	    fcntl(__svgalib_mouse_fd, F_SETFL, O_RDONLY);
 	    m_fdmode = 1;
-#ifdef BACKGROUND
-            /* Waits while foreground again. No reading in background. */
-	    while(!vga_oktowrite()) sleep(0);
-#endif
 	    read(__svgalib_mouse_fd, &buf[nu_bytes], 1);
             if ((m_type == MOUSE_SPACEBALL)) {
               nu_packets=(buf[nu_bytes]==13);
@@ -750,7 +744,19 @@ static int get_ms_event(int wait) {
 	    }
     /* Construct the event. */
     switch (m_type) {
+#ifdef MS3B
     case MOUSE_MICROSOFT:	/* Microsoft */
+        but = (but & 8) | ((buf[i] & 0x20) >> 3) | ((buf[i] & 0x10) >> 4);
+	dx = (char) (((buf[i] & 0x03) << 6) | (buf[i + 1] & 0x3F));
+	dy = (char) (((buf[i] & 0x0C) << 4) | (buf[i + 2] & 0x3F));
+        if((dx==0)&&(dy==0)&&(but==(prev&~MOUSE_MIDDLEBUTTON)))
+            but=prev^MOUSE_MIDDLEBUTTON; else
+            but |= prev&MOUSE_MIDDLEBUTTON;
+        prev=but;
+        break;
+#else
+    case MOUSE_MICROSOFT:	/* Microsoft */
+#endif
     case MOUSE_LOGIMAN:	/* MouseMan / TrackMan */
     case MOUSE_PNP:
     default:
