@@ -12,7 +12,6 @@ when entering text mode) or to the et4000 autodetection.
  
 */
 
-
 #include <stdlib.h>
 #include <stdio.h>		/* for printf */
 #include <string.h>		/* for memset */
@@ -30,8 +29,15 @@ when entering text mode) or to the et4000 autodetection.
 #include "vgapci.h"
 
 #define MXREG_SAVE(i) (VGA_TOTAL_REGS+i)
-#define MX_TOTAL_REGS (VGA_TOTAL_REGS + 23)
+#define MX_TOTAL_REGS (VGA_TOTAL_REGS + 36)
 
+static struct {
+    unsigned char c8;
+    unsigned short c15;
+    unsigned short c16;
+    unsigned int c32;
+} cursor_colors[16*2];
+static unsigned char *cursors[16];
 
 static int mx_init(int, int, int);
 static void mx_unlock(void);
@@ -49,6 +55,29 @@ enum {
    	MX86250 = 0, MX86251
 };
 
+static int pal=0, palette[768];
+
+static int findcolor(int rgb) {
+   int i,j,k,l;
+
+   if(pal)vga_getpalvec(0,256,palette);
+   pal=0;
+   k=0xffffff;
+   for(i=0;i<256;i++) {
+      j=((rgb&0xff)-(palette[i*3+2]<<2))*((rgb&0xff)-(palette[i*3+2]<<2))+
+        (((rgb>>8)&0xff)-(palette[i*3+1]<<2))*(((rgb>>8)&0xff)-(palette[i*3+1]<<2))+
+        (((rgb>>16)&0xff)-(palette[i*3]<<2))*(((rgb>>16)&0xff)-(palette[i*3]<<2));
+      if(j==0) {
+         return i;
+      }
+      if(j<k) {
+         k=j;
+         l=i;
+      }
+   }
+   return l;
+}
+
 static void mx_setpage(int page)
 {
   __svgalib_outCR(BankIdx,page);
@@ -106,6 +135,10 @@ static int mx_saveregs(unsigned char regs[])
         regs[MXREG_SAVE(9)]=__svgalib_inCR(0x20);
       break;
     }
+
+    for(i=0x60;i<0x6d;i++) if(i!=0x66)
+        regs[MXREG_SAVE(i-0x60+23)]=__svgalib_inCR(i);
+    
     return MX_TOTAL_REGS - VGA_TOTAL_REGS;
 }
 
@@ -139,7 +172,10 @@ static void mx_setregs(const unsigned char regs[], int mode)
         for(i=0x1f;i<0x21;i++)__svgalib_outCR(i,regs[MXREG_SAVE(i-0x1f+8)]);
       break;
     }
-    
+
+    for(i=0x60;i<0x6d;i++) if(i!=0x66)
+        __svgalib_outCR(i,regs[MXREG_SAVE(i-0x60+23)]);
+
     __svgalib_outSR(0,3);
 }
 
@@ -291,37 +327,29 @@ static void mx_initializemode(unsigned char *moderegs,
 	    }
     }
 
-{ int p,q,r;
-
-p=moderegs[1]+((moderegs[MXREG_SAVE(1)]&2)<<7);
-moderegs[2]=p&0xff;
-/*
-moderegs[MXREG_SAVE(1)]&=0xfd;
-moderegs[MXREG_SAVE(1)]|=(p&0x100)>>7;
-*/
-q=moderegs[4]+((moderegs[MXREG_SAVE(1)]&8)<<5);
-r=moderegs[0]+4;
-moderegs[3]&=0xe0;
-moderegs[3]|=(r&0x1f);
-moderegs[5]&=0x7f;
-moderegs[5]|=(r&0x20)<<2;
-
-p=moderegs[0x12]+((moderegs[0x7]&2)<<7)+((moderegs[7]&0x40)<<3)+((moderegs[MXREG_SAVE(2)]&2)<<9);
-moderegs[0x15]=p&0xff;
-moderegs[0x7]&=0xf7;
-moderegs[0x7]|=(p&0x100)>>5;
-moderegs[0x9]&=0xdf;
-moderegs[0x9]|=(p&0x200)>>4;
-/*
-moderegs[MXREG_SAVE(2)]&=0xfb;
-moderegs[MXREG_SAVE(2)]|=(p&0x400)>>7;
-*/
-moderegs[0x16]=moderegs[6]+1;
-}
-
-mx_is_linear=0;
-
-return ;
+    { int p,q,r;
+    
+    p=moderegs[1]+((moderegs[MXREG_SAVE(1)]&2)<<7);
+    moderegs[2]=p&0xff;
+    q=moderegs[4]+((moderegs[MXREG_SAVE(1)]&8)<<5);
+    r=moderegs[0]+4;
+    moderegs[3]&=0xe0;
+    moderegs[3]|=(r&0x1f);
+    moderegs[5]&=0x7f;
+    moderegs[5]|=(r&0x20)<<2;
+    
+    p=moderegs[0x12]+((moderegs[0x7]&2)<<7)+((moderegs[7]&0x40)<<3)+((moderegs[MXREG_SAVE(2)]&2)<<9);
+    moderegs[0x15]=p&0xff;
+    moderegs[0x7]&=0xf7;
+    moderegs[0x7]|=(p&0x100)>>5;
+    moderegs[0x9]&=0xdf;
+    moderegs[0x9]|=(p&0x200)>>4;
+    moderegs[0x16]=moderegs[6]+1;
+    }
+    
+    mx_is_linear=0;
+    
+    return ;
 
 }
 
@@ -460,6 +488,105 @@ static int mx_map_horizontal_crtc(int bpp, int pixelclock, int htiming)
 return htiming;
 }
 
+static int mx_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
+    unsigned long *b3;
+    int i, j, k, l; 
+    unsigned int l1, l2;
+    unsigned char c;
+    
+    switch(cmd){
+        case CURSOR_INIT:
+            for (i=0;i<16;i++)cursors[i]=NULL;
+            return 1;
+        case CURSOR_HIDE:
+            __svgalib_outcrtc(0x66,__svgalib_incrtc(0x66)&0xfe);
+            break;
+        case CURSOR_SHOW:
+            __svgalib_outcrtc(0x66,__svgalib_incrtc(0x66)|1);
+            break;
+        case CURSOR_POSITION:
+            if(CI.colors>65536)p1&= ~1; /* the X server does this */
+            __svgalib_outcrtc(0x61,p1>>8);
+            __svgalib_outcrtc(0x60,p1&0xff);
+            __svgalib_outcrtc(0x63,p2&0xff);
+            __svgalib_outcrtc(0x62,0);
+            __svgalib_outcrtc(0x65,0);
+            __svgalib_outcrtc(0x64,p2>>8);
+            break;
+        case CURSOR_IMAGE:
+            switch(p2){
+                case 0:
+                    if(cursors[p1]!=NULL) {
+                        free(cursors[p1]);
+                    }
+                    cursors[p1]=malloc(256);
+                    memcpy(cursors[p1],p5,256);
+                    cursor_colors[p1*2].c8=findcolor(p3);
+                    cursor_colors[p1*2].c32=p3;
+                    cursor_colors[p1*2].c16=((p3&0xf80000)>>8)|((p3&0xfc00)>>5)|((p3&0xf8)>>3);
+                    cursor_colors[p1*2].c15=((p3&0xf80000)>>9)|((p3&0xf800)>>5)|((p3&0xf8)>>3);
+                    cursor_colors[p1*2+1].c8=findcolor(p4);
+                    cursor_colors[p1*2+1].c32=p4;
+                    cursor_colors[p1*2+1].c16=((p4&0xf80000)>>8)|((p4&0xfc00)>>5)|((p4&0xf8)>>3);
+                    cursor_colors[p1*2+1].c15=((p4&0xf80000)>>9)|((p4&0xf800)>>5)|((p4&0xf8)>>3);
+                    break;
+            }
+        case CURSOR_SELECT:
+            i=mx_memory*1024-256;
+            b3=(unsigned long *)cursors[p1];
+            switch(p2) {
+                case 0:
+                    switch(CI.colors) {
+                        case 256:
+                           __svgalib_outcrtc(0x6a,cursor_colors[p1*2+1].c8);
+                           __svgalib_outcrtc(0x67,cursor_colors[p1*2].c8);
+                           break;
+                        case 32768:
+                           __svgalib_outcrtc(0x6a,cursor_colors[p1*2+1].c15>>8);
+                           __svgalib_outcrtc(0x6b,cursor_colors[p1*2+1].c15&0xff);
+                           __svgalib_outcrtc(0x67,cursor_colors[p1*2].c15>>8);
+                           __svgalib_outcrtc(0x68,cursor_colors[p1*2].c15&0xff);
+                           break;
+                        case 65536:
+                           __svgalib_outcrtc(0x6a,cursor_colors[p1*2+1].c16>>8);
+                           __svgalib_outcrtc(0x6b,cursor_colors[p1*2+1].c16&0xff);
+                           __svgalib_outcrtc(0x67,cursor_colors[p1*2].c16>>8);
+                           __svgalib_outcrtc(0x68,cursor_colors[p1*2].c16&0xff);
+                           break;
+                        case (1<<24):
+                           __svgalib_outcrtc(0x6a,cursor_colors[p1*2+1].c32>>16);
+                           __svgalib_outcrtc(0x6b,(cursor_colors[p1*2+1].c32>>8)&0xff);
+                           __svgalib_outcrtc(0x6c,cursor_colors[p1*2+1].c32&0xff);
+                           __svgalib_outcrtc(0x67,cursor_colors[p1*2].c32>>16);
+                           __svgalib_outcrtc(0x68,(cursor_colors[p1*2].c32>>8)&0xff);
+                           __svgalib_outcrtc(0x69,cursor_colors[p1*2].c32&0xff);
+                           break;
+                    }
+                        
+                    for(j=0;j<32;j++) {
+                        l1=*(b3+j);
+                        l2=~(*(b3+32+j));
+                        for(k=0;k<8;k++) {
+                            c=0;
+                            for(l=0;l<4;l++) {
+                                c<<=1;
+                                c|=(l2>>31);
+                                c<<=1;
+                                c|=(l1>>31);
+                                if(c&2)c&=0xfe;
+                                l1<<=1;
+                                l2<<=1;
+                            }
+                            *(LINEAR_MEM_POINTER+i+8*j+k)=c;
+                        }
+                    }
+                    break;
+            }
+            break;
+    }
+    return 0;
+}       
+
 /* Function table (exported) */
 
 DriverSpecs __svgalib_mx_driverspecs =
@@ -488,6 +615,7 @@ DriverSpecs __svgalib_mx_driverspecs =
     mx_linear,
     0,				/* accelspecs, filled in during init. */
     NULL,                       /* Emulation */
+    mx_cursor,
 };
 
 /* Initialize chipset (called after detection) */
