@@ -13,7 +13,6 @@ there are still the following problems:
   * no acceleration (is there a program that uses it anyway?).
 */
 
-
 #include <stdlib.h>
 #include <stdio.h>		/* for printf */
 #include <string.h>		/* for memset */
@@ -25,7 +24,6 @@ there are still the following problems:
 #include "libvga.h"
 #include "driver.h"
 
-
 /* New style driver interface. */
 #include "timing.h"
 #include "vgaregs.h"
@@ -47,6 +45,8 @@ there are still the following problems:
 #define SetBit(n) (1<<(n))
 #define Set8Bits(value) ((value)&0xff)
 
+#define GET15BITCOLOR(r,g,b) ((((r)&0xf8)<<7)|(((g)&0xf8)<<2)|((b)>>3))
+
 static volatile unsigned  *nvPFBPort;
 static volatile unsigned  *nvPRAMDACPort;
 static volatile unsigned  *nvPEXTDEVPort;
@@ -119,31 +119,35 @@ static int nv3_saveregs(unsigned char regs[])
     regs[NV3REG_SAVE(6)] = __svgalib_incrtc(NV_PCRTC_FIFO);
     regs[NV3REG_SAVE(7)] = 0;
 
-
-   k = PFB_Read(CONFIG_0) ;
-   regs[NV3REG_SAVE(8)] = k&0xff;
-   regs[NV3REG_SAVE(9)] = (k >> 8)&0xff;
-   regs[NV3REG_SAVE(10)] = (k >> 16) & 0xff ;
-   regs[NV3REG_SAVE(11)] = ( k >> 24 ) & 0xff ;
-   k =  PRAMDAC_Read(VPLL_COEFF);
-   regs[NV3REG_SAVE(12)] = k&0xff;
-   regs[NV3REG_SAVE(13)] = (k >> 8)&0xff;
-   regs[NV3REG_SAVE(14)] = (k >> 16) & 0xff ;
-   regs[NV3REG_SAVE(15)] = ( k >> 24 ) & 0xff ;
-   k =  PRAMDAC_Read(PLL_COEFF_SELECT);
-   regs[NV3REG_SAVE(16)] = k&0xff;
-   regs[NV3REG_SAVE(17)] = (k >> 8)&0xff;
-   regs[NV3REG_SAVE(18)] = (k >> 16) & 0xff ;
-   regs[NV3REG_SAVE(19)] = ( k >> 24 ) & 0xff ;
-   k =  PRAMDAC_Read(GENERAL_CONTROL);
-   regs[NV3REG_SAVE(20)] = k&0xff;
-   regs[NV3REG_SAVE(21)] = (k >> 8)&0xff;
-   regs[NV3REG_SAVE(22)] = (k >> 16) & 0xff ;
-   regs[NV3REG_SAVE(23)] = ( k >> 24 ) & 0xff ;
-   
-   for(i=0x18;i<0x38;i++)regs[84+i-0x18]=__svgalib_incrtc(i);
-   
-   return NV3_TOTAL_REGS - VGA_TOTAL_REGS;
+    k = PFB_Read(CONFIG_0) ;
+    regs[NV3REG_SAVE(8)] = k&0xff;
+    regs[NV3REG_SAVE(9)] = (k >> 8)&0xff;
+    regs[NV3REG_SAVE(10)] = (k >> 16) & 0xff ;
+    regs[NV3REG_SAVE(11)] = ( k >> 24 ) & 0xff ;
+    k =  PRAMDAC_Read(VPLL_COEFF);
+    regs[NV3REG_SAVE(12)] = k&0xff;
+    regs[NV3REG_SAVE(13)] = (k >> 8)&0xff;
+    regs[NV3REG_SAVE(14)] = (k >> 16) & 0xff ;
+    regs[NV3REG_SAVE(15)] = ( k >> 24 ) & 0xff ;
+    k =  PRAMDAC_Read(PLL_COEFF_SELECT);
+    regs[NV3REG_SAVE(16)] = k&0xff;
+    regs[NV3REG_SAVE(17)] = (k >> 8)&0xff;
+    regs[NV3REG_SAVE(18)] = (k >> 16) & 0xff ;
+    regs[NV3REG_SAVE(19)] = ( k >> 24 ) & 0xff ;
+    k =  PRAMDAC_Read(GENERAL_CONTROL);
+    regs[NV3REG_SAVE(20)] = k&0xff;
+    regs[NV3REG_SAVE(21)] = (k >> 8)&0xff;
+    regs[NV3REG_SAVE(22)] = (k >> 16) & 0xff ;
+    regs[NV3REG_SAVE(23)] = ( k >> 24 ) & 0xff ;
+    k =  PRAMDAC_Read(GRCURSOR_START_POS);
+    regs[NV3REG_SAVE(56)] = k&0xff;
+    regs[NV3REG_SAVE(57)] = (k >> 8)&0xff;
+    regs[NV3REG_SAVE(58)] = (k >> 16) & 0xff ;
+    regs[NV3REG_SAVE(59)] = ( k >> 24 ) & 0xff ;
+    
+    for(i=0x18;i<0x38;i++)regs[84+i-0x18]=__svgalib_incrtc(i);
+    
+    return NV3_TOTAL_REGS - VGA_TOTAL_REGS;
 }
 
 /* Set chipset-specific registers */
@@ -163,7 +167,8 @@ static void nv3_setregs(const unsigned char regs[], int mode)
    __svgalib_outcrtc(0x1c,regs[88]); /* this enables banking at 0xa0000 */
    __svgalib_outcrtc(0x1d,regs[89]); 
    __svgalib_outcrtc(0x1e,regs[90]); 
-   __svgalib_outcrtc(0x31,0); /* disable cursor */
+   __svgalib_outcrtc(0x30,regs[108]); 
+   __svgalib_outcrtc(0x31,regs[109]); 
 
    k=regs[NV3REG_SAVE(8)]+(regs[NV3REG_SAVE(9)] << 8)+(regs[NV3REG_SAVE(10)] << 16)+(regs[NV3REG_SAVE(11)] << 24); 
    PFB_Write(CONFIG_0,k);
@@ -174,6 +179,8 @@ static void nv3_setregs(const unsigned char regs[], int mode)
    PRAMDAC_Write(PLL_COEFF_SELECT,k);
    k=regs[NV3REG_SAVE(20)]+(regs[NV3REG_SAVE(21)] << 8)+(regs[NV3REG_SAVE(22)] << 16)+(regs[NV3REG_SAVE(23)] << 24);
    PRAMDAC_Write(GENERAL_CONTROL,k);
+   k=regs[NV3REG_SAVE(56)]+(regs[NV3REG_SAVE(57)] << 8)+(regs[NV3REG_SAVE(58)] << 16)+(regs[NV3REG_SAVE(59)] << 24);
+   PRAMDAC_Write(GRCURSOR_START_POS,k);
 }
 
 
@@ -348,10 +355,6 @@ static void nv3_initializemode(unsigned char *moderegs,
    moderegs[NV3REG_SAVE(18)] = (k >> 16) & 0xff ;
    moderegs[NV3REG_SAVE(19)] = ( k >> 24 ) & 0xff ;
 
-  /* Disable Tetris tiling for now. This looks completely mad but could 
-   * give some significant performance gains. Will perhaps experiment 
-   * later on with this stuff!  
-   */
    k=
       PFB_Val(CONFIG_0_RESOLUTION,((modeinfo->lineWidth+31)/32))|
       PFB_Val(CONFIG_0_PIXEL_DEPTH,pixelDepth)|
@@ -361,11 +364,12 @@ static void nv3_initializemode(unsigned char *moderegs,
    moderegs[NV3REG_SAVE(10)] = (k >> 16) & 0xff ;
    moderegs[NV3REG_SAVE(11)] = ( k >> 24 ) & 0xff ;
 
-  moderegs[88]=28;
+   moderegs[88]=28;
+   moderegs[109]&=0xfe; /* hide cursor */
 
-  if(modeinfo->greenWeight==5){moderegs[NV3REG_SAVE(21)]&=0xef;}; 
+   if(modeinfo->greenWeight==5){moderegs[NV3REG_SAVE(21)]&=0xef;}; 
 
-  nv3_is_linear=0;
+   nv3_is_linear=0;
 
 return ;
 }
@@ -413,7 +417,6 @@ static int nv3_setmode(int mode, int prv_mode)
     return 0;
 }
 
-
 /* Unlock chipset-specific registers */
 
 static void nv3_unlock(void)
@@ -428,7 +431,6 @@ static void nv4_unlock(void)
     __svgalib_nv3_outcrtc(0x1f, UNLOCK_EXT_MAGIC);
 }
 
-
 /* Relock chipset-specific registers */
 /* (currently not used) */
 
@@ -483,7 +485,6 @@ static void nv3_setwrpage(int page)
 {
 }
 
-
 /* Set display start address (not for 16 color modes) */
 
 static void nv3_setdisplaystart(int address)
@@ -496,7 +497,6 @@ static void nv3_setdisplaystart(int address)
   
 }
 
-
 /* Set logical scanline length (usually multiple of 8) */
 
 static void nv3_setlogicalwidth(int width)
@@ -516,6 +516,71 @@ if (op == LINEAR_QUERY_RANGE || op == LINEAR_QUERY_GRANULARITY) return 0;		/* No
     else return -1;		/* Unknown function. */
 }
 
+static int nv3_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
+    unsigned char *b1, *b2;
+    unsigned short *b3;
+    unsigned int i, j, k, l, c0, c1;
+    
+    switch(cmd){
+        case CURSOR_INIT:
+            return 1;
+        case CURSOR_HIDE:
+            __svgalib_outcrtc(0x31,__svgalib_incrtc(0x31)&0xfe); /* disable cursor */
+            break;
+        case CURSOR_SHOW:
+            __svgalib_outcrtc(0x31,__svgalib_incrtc(0x31)|1); /* enable cursor */
+            break;
+        case CURSOR_POSITION:
+            PRAMDAC_Write(GRCURSOR_START_POS,p1+(p2<<16));
+            break;
+        case CURSOR_SELECT:
+            i=nv3_memory/2-(p1+1);
+            if (nv3_chiptype==Riva128) {
+                __svgalib_outcrtc(0x31,(__svgalib_incrtc(0x31)&7)|(((~i)&0x1f)<<3));
+            } else {
+                __svgalib_outcrtc(0x31,(__svgalib_incrtc(0x31)&3)|(((~i)&0x3f)<<2));
+            }
+#if 0
+            __svgalib_outcrtc(0x30,((~i)&0x3fc0)>>6); 
+#else
+            __svgalib_outcrtc(0x30,0); 
+#endif
+            break;
+        case CURSOR_IMAGE:
+            i=nv3_memory/2-(p1+1);
+            i=i*2048;
+            switch(p2) {
+                case 0: /* X11 format, 32x32 */
+                    b3=malloc(2048);
+                    b1=(unsigned char *)p5;
+                    b2=b1+128;
+                    c0=0x8000|GET15BITCOLOR((p3>>16)&0xff,(p3>>8)&0xff,p3&0xff);
+                    c1=0x8000|GET15BITCOLOR((p4>>16)&0xff,(p4>>8)&0xff,p4&0xff);
+                    l=992;
+                    for(k=0;k<128;k++) {
+                        int cc1=*(b1+k);
+                        int cc2=*(b2+k);
+                        for(j=0;j<8;j++) {
+                            if(!(cc2&0x80)) *(b3+l)=0; else if (cc1&0x80)
+                                *(b3+l)=c1; else *(b3+l)=c0;
+                            l++;
+                            if((l&0x1f)==0)l-=64;
+                            cc2<<=1;
+                            cc1<<=1;
+                        }
+                    }
+                    memcpy(LINEAR_MEM_POINTER+i,b3,2048);
+                    free(b3);
+                    break;
+                case 1: /* nvidia 1555 format  32x32 */
+                    memcpy(LINEAR_MEM_POINTER+i,p5,2048);
+                    break;
+            }
+            break;
+    }
+    return 0;
+}       
+
 static int nv3_match_programmable_clock(int clock)
 {
 return clock ;
@@ -554,7 +619,9 @@ DriverSpecs __svgalib_nv3_driverspecs =
     0,				/* ext_set */
     0,				/* accel */
     nv3_linear,
-    0				/* accelspecs, filled in during init. */
+    0,				/* accelspecs, filled in during init. */
+    0,
+    nv3_cursor
 };
 
 #define MapDevice(device,base) \
