@@ -43,11 +43,6 @@ Copyright 1994 through 1998 by Marc Aurele La France (TSI @ UQV), tsi@ualberta.c
 #include "vgapci.h"
 #include "rage.h"
 
-/*
-#define RAGE_TOTAL_REGS (VGA_TOTAL_REGS + 64)
-*/
-#define RAGE_TOTAL_REGS (VGA_TOTAL_REGS + 1000)
-
 int           ATIIOPortCRTC_H_TOTAL_DISP, ATIIOPortCRTC_H_SYNC_STRT_WID,
               ATIIOPortCRTC_V_TOTAL_DISP, ATIIOPortCRTC_V_SYNC_STRT_WID,
               ATIIOPortCRTC_OFF_PITCH,ATIIOPortCRTC_INT_CNTL,
@@ -59,7 +54,9 @@ int           ATIIOPortCRTC_H_TOTAL_DISP, ATIIOPortCRTC_H_SYNC_STRT_WID,
               ATIIOPortDAC_REGS, ATIIOPortDAC_CNTL, ATIIOPortDAC_DATA,
               ATIIOPortDAC_READ, ATIIOPortDAC_WRITE,ATIIOPortDAC_MASK,
               ATIIOPortGEN_TEST_CNTL, ATIIOPortCONFIG_CNTL,
-              ATIIOPortCONFIG_STATUS64_0;
+              ATIIOPortCONFIG_STATUS64_0, ATIIOPortCUR_HORZ_VERT_OFF,
+              ATIIOPortCUR_HORZ_VERT_POSN, ATIIOPortCUR_CLR0,
+              ATIIOPortCUR_CLR1, ATIIOPortCUR_OFFSET;
 
 void ATIAccessMach64PLLReg(const int Index, const int Write)
 {
@@ -87,12 +84,16 @@ typedef struct {
                mem_info,
                mem_buf_cntl, shared_cntl, shared_mem_config,
                crtc_int_cntl, gen_test_cntl,
-               misc_options, mem_bndry, mem_cfg;
+               misc_options, mem_bndry, mem_cfg,
+               cur_clr0, cur_clr1, cur_offset, cur_horz_vert_posn, cur_horz_vert_off;
         unsigned char PLL[32];
         unsigned char DAC[DAC_SIZE];
         unsigned char extdac[16];
         int ics2595;
        } ATIHWRec, *ATIHWPtr;
+
+#define RAGE_TOTAL_REGS (VGA_TOTAL_REGS + sizeof(ATIHWRec))
+
 extern unsigned char __svgalib_ragedoubleclock;
 static int rage_init(int, int, int);
 static void rage_unlock(void);
@@ -389,32 +390,6 @@ int rage_saveregs(unsigned char regs[])
 
   save=(ATIHWPtr)(regs+VGA_TOTAL_REGS);
 
-/* This are needed for calling this function, without vga_init first */
-/*    
-    ATIIODecoding=1;
-    ATIIOBase=0xd000;
-    ATIIOPortCRTC_H_TOTAL_DISP=ATIIOPort(CRTC_H_TOTAL_DISP);
-    ATIIOPortCRTC_H_SYNC_STRT_WID=ATIIOPort(CRTC_H_SYNC_STRT_WID);
-    ATIIOPortCRTC_V_TOTAL_DISP=ATIIOPort(CRTC_V_TOTAL_DISP);
-    ATIIOPortCRTC_V_SYNC_STRT_WID=ATIIOPort(CRTC_V_SYNC_STRT_WID);
-    ATIIOPortCRTC_OFF_PITCH=ATIIOPort(CRTC_OFF_PITCH);
-    ATIIOPortCRTC_INT_CNTL=ATIIOPort(CRTC_INT_CNTL);
-    ATIIOPortCRTC_GEN_CNTL=ATIIOPort(CRTC_GEN_CNTL);
-    ATIIOPortDSP_CONFIG=ATIIOPort(DSP_CONFIG);
-    ATIIOPortDSP_ON_OFF=ATIIOPort(DSP_ON_OFF);
-    ATIIOPortOVR_CLR=ATIIOPort(OVR_CLR);
-    ATIIOPortOVR_WID_LEFT_RIGHT=ATIIOPort(OVR_WID_LEFT_RIGHT);
-    ATIIOPortOVR_WID_TOP_BOTTOM=ATIIOPort(OVR_WID_TOP_BOTTOM);
-    ATIIOPortCLOCK_CNTL=ATIIOPort(CLOCK_CNTL);
-    ATIIOPortBUS_CNTL=ATIIOPort(BUS_CNTL);
-    ATIIOPortMEM_INFO=ATIIOPort(MEM_INFO);
-    ATIIOPortMEM_VGA_WP_SEL=ATIIOPort(MEM_VGA_WP_SEL);
-    ATIIOPortMEM_VGA_RP_SEL=ATIIOPort(MEM_VGA_RP_SEL);
-    ATIIOPortDAC_REGS=ATIIOPort(DAC_REGS);
-    ATIIOPortDAC_CNTL=ATIIOPort(DAC_CNTL);
-    ATIIOPortGEN_TEST_CNTL=ATIIOPort(GEN_TEST_CNTL);
-    ATIIOPortCONFIG_CNTL=ATIIOPort(CONFIG_CNTL);
-*/
   save->crtc_gen_cntl = inl(ATIIOPortCRTC_GEN_CNTL);
 
   save->crtc_h_total_disp = inl(ATIIOPortCRTC_H_TOTAL_DISP);
@@ -444,6 +419,12 @@ int rage_saveregs(unsigned char regs[])
   save->crtc_gen_cntl=inl(ATIIOPortCRTC_GEN_CNTL);
   save->gen_test_cntl=inl(ATIIOPortGEN_TEST_CNTL);  
 
+  save->cur_clr0=inl(ATIIOPortCUR_CLR0);
+  save->cur_clr1=inl(ATIIOPortCUR_CLR1);
+  save->cur_offset=inl(ATIIOPortCUR_OFFSET);
+  save->cur_horz_vert_posn=inl(ATIIOPortCUR_HORZ_VERT_POSN);
+  save->cur_horz_vert_off=inl(ATIIOPortCUR_HORZ_VERT_OFF);
+
   switch(rage_clock){
      case 0: /* internal clock */
         for(i=6;i<12;i++)
@@ -532,6 +513,12 @@ static void rage_setregs(const unsigned char regs[], int mode)
     outl(ATIIOPortCRTC_GEN_CNTL,restore->crtc_gen_cntl);
     outl(ATIIOPortGEN_TEST_CNTL,restore->gen_test_cntl);
 
+    outl(ATIIOPortCUR_CLR0, restore->cur_clr0);
+    outl(ATIIOPortCUR_CLR1, restore->cur_clr1);
+    outl(ATIIOPortCUR_OFFSET, restore->cur_offset);
+    outl(ATIIOPortCUR_HORZ_VERT_POSN, restore->cur_horz_vert_posn);
+    outl(ATIIOPortCUR_HORZ_VERT_OFF, restore->cur_horz_vert_off);
+
     switch(rage_clock){
        case 0: /* internal clock */
           i=ATIGetMach64PLLReg(PLL_VCLK_CNTL) | PLL_VCLK_RESET;
@@ -1034,21 +1021,21 @@ static int rage_test(void)
 
 static void rage_setdisplaystart(int address)
 { 
-unsigned int t;
-
-address>>=3;
-t=inl(ATIIOPortCRTC_OFF_PITCH);
-outl(ATIIOPortCRTC_OFF_PITCH,(t&~CRTC_OFFSET)|address);
+    unsigned int t;
+    
+    address>>=3;
+    t=inl(ATIIOPortCRTC_OFF_PITCH);
+    outl(ATIIOPortCRTC_OFF_PITCH,(t&~CRTC_OFFSET)|address);
 
 }
 
 static void rage_setlogicalwidth(int width)
 {   
-unsigned int t;
-
-if(rage_bpp>0)width=width/rage_bpp;
-t=inl(ATIIOPortCRTC_OFF_PITCH);
-outl(ATIIOPortCRTC_OFF_PITCH,(t&~CRTC_PITCH)|((width>>3)<<22));
+    unsigned int t;
+    
+    if(rage_bpp>0)width=width/rage_bpp;
+    t=inl(ATIIOPortCRTC_OFF_PITCH);
+    outl(ATIIOPortCRTC_OFF_PITCH,(t&~CRTC_PITCH)|((width>>3)<<22));
 
 }
 
@@ -1075,8 +1062,68 @@ static int rage_map_horizontal_crtc(int bpp, int pixelclock, int htiming)
 return htiming;
 }
 
-/* Function table (exported) */
+static unsigned int cur_colors[16*2];
 
+static int rage_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
+    int i, j, t;
+    unsigned long *b3;
+    unsigned long l1, l2, k, c, l;
+    
+    switch(cmd){
+        case CURSOR_INIT:
+            return 1;
+        case CURSOR_HIDE:
+            t=inl(ATIIOPortGEN_TEST_CNTL);
+            outl(ATIIOPortGEN_TEST_CNTL,t&~GEN_CUR_EN);
+            break;
+        case CURSOR_SHOW:
+            t=inl(ATIIOPortGEN_TEST_CNTL);
+            outl(ATIIOPortGEN_TEST_CNTL,t|GEN_CUR_EN);
+            break;
+        case CURSOR_POSITION:
+            outl(ATIIOPortCUR_HORZ_VERT_POSN,p1|(p2<<16));
+            outl(ATIIOPortCUR_HORZ_VERT_OFF,0);
+        break;
+        case CURSOR_SELECT:
+            i=rage_memory*1024-(p1+1)*4096;
+            outl(ATIIOPortCUR_OFFSET,i>>3);
+            outl(ATIIOPortCUR_CLR0,cur_colors[p1*2]);
+            outl(ATIIOPortCUR_CLR1,cur_colors[p1*2+1]);
+            break;
+        case CURSOR_IMAGE:
+            i=rage_memory*1024-(p1+1)*4096;
+            b3=(unsigned long *)p5;
+            switch(p2) {
+                case 0:
+                    cur_colors[p1*2]=p3<<8;
+                    cur_colors[p1*2+1]=p4<<8;
+                    for(j=0;j<32;j++) {
+                        l1=*(b3+j);
+                        l2=~(*(b3+32+j));
+                        for(k=0;k<8;k++) {
+                            c=0;
+                            for(l=0;l<4;l++) {
+                                c>>=1;
+                                c|=(l1>>31)<<7;
+                                c>>=1;
+                                c|=(l2>>31)<<7;
+                                if(c&128)c&=0xbf;
+                                l1<<=1;
+                                l2<<=1;
+                            }
+                            *(LINEAR_MEM_POINTER+i+16*j+k)=c;
+                            *(LINEAR_MEM_POINTER+i+16*j+k+8)=0xaa;
+                        }
+                    }
+                    memset(LINEAR_MEM_POINTER+i+512,0xaa,512);
+                break;
+            }
+            break;
+    }
+    return 0;
+}       
+
+/* Function table (exported) */
 DriverSpecs __svgalib_rage_driverspecs =
 {
     rage_saveregs,
@@ -1103,6 +1150,7 @@ DriverSpecs __svgalib_rage_driverspecs =
     rage_linear,
     0,				/* accelspecs, filled in during init. */
     NULL,                       /* Emulation */
+    rage_cursor
 };
 
 /* Initialize chipset (called after detection) */
@@ -1180,6 +1228,11 @@ static int rage_init(int force, int par1, int par2)
    ATIIOPortGEN_TEST_CNTL=ATIIOPort(GEN_TEST_CNTL);
    ATIIOPortCONFIG_CNTL=ATIIOPort(CONFIG_CNTL);
    ATIIOPortCONFIG_STATUS64_0=ATIIOPort(CONFIG_STATUS64_0);
+   ATIIOPortCUR_HORZ_VERT_OFF=ATIIOPort(CUR_HORZ_VERT_OFF);
+   ATIIOPortCUR_HORZ_VERT_POSN=ATIIOPort(CUR_HORZ_VERT_POSN);
+   ATIIOPortCUR_CLR0=ATIIOPort(CUR_CLR0);
+   ATIIOPortCUR_CLR1=ATIIOPort(CUR_CLR1);
+   ATIIOPortCUR_OFFSET=ATIIOPort(CUR_OFFSET);
 
    i = inl(ATIIOPort(MEM_INFO));
    j = inl(ATIIOPort(CONFIG_STATUS64_0));
