@@ -6,7 +6,6 @@ Matan Ziv-Av <matan@svgalib.org>
 Fixes for 530 and for vesafb by Marcelo de Paula Bezerra 
 <mosca@internetaddress.com>.
 
-
 This driver is based on the XFree86 sis driver, written by
 
  Alan Hourihane
@@ -66,6 +65,7 @@ enum { SIS_86C201=0, SIS_86C202, SIS_86C205, SIS_86C215, SIS_86C225,
 
 #define write_xr(num,val) {__svgalib_outseq(num,val);}
 #define read_xr(num,var) {var=__svgalib_inseq(num);} 
+#define in_xr(num) __svgalib_inseq(num)
 
 static int sis_init(int, int, int);
 static void sis_unlock(void);
@@ -160,7 +160,6 @@ static void sis_setregs(const unsigned char regs[], int mode)
 
 }
 
-
 /* Return nonzero if mode is available */
 
 static int sis_modeavailable(int mode)
@@ -302,6 +301,9 @@ static void sis_initializemode(unsigned char *moderegs,
 
     moderegs[59]=0x6f;
 
+    moderegs[XR(0x1c)]=0;
+    moderegs[XR(0x1f)]=0;
+
     return ;
 }
 
@@ -458,6 +460,82 @@ static int sis_linear(int op, int param)
         else return -1;		/* Unknown function. */
 }
 
+static unsigned char cur_colors[16*6];
+
+static int sis_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
+    unsigned long *b3;
+    int i, j, k, l; 
+    unsigned int l1, l2;
+    unsigned char c;
+    
+    switch(cmd){
+        case CURSOR_INIT:
+            return 1;
+        case CURSOR_HIDE:
+            write_xr(6,in_xr(6)&0xbf);
+            break;
+        case CURSOR_SHOW:
+            write_xr(6,in_xr(6)|0x40);
+            break;
+        case CURSOR_POSITION:
+            write_xr(0x1a,p1&0xff);
+            write_xr(0x1b,p1>>8);
+            p2>>=1;
+            write_xr(0x1d,p2&0xff);
+            write_xr(0x1e,(in_xr(0x1e)&0xf8)|((p2>>8)&0x07));
+            break;
+        case CURSOR_SELECT:
+            i=sis_memory*1024-16384;
+            if((sis_chiptype==SIS_6326)&&(sis_memory>4096))
+               i-=4096*1024; /* 6326 can't put cursor beyond 4MB */
+            for(j=0;j<6;j++)write_xr(0x14+j,cur_colors[p1*6+j]>>2);
+            write_xr(0x1e,(in_xr(0x1e)&0x0f)|(p1<<4));
+            write_xr(0x38,(in_xr(0x38)&0x0f)|((i>>18)<<4));
+            if(i&0x400000) {
+               write_xr(0x3e,in_xr(0x3e)|0x04);
+            } else {
+               write_xr(0x3e,in_xr(0x3e)&0xfb);
+            }
+            break;
+        case CURSOR_IMAGE:
+            i=sis_memory*1024-(16-p1)*1024;
+            if((sis_chiptype==SIS_6326)&&(sis_memory>4096))
+               i-=4096*1024; /* 6326 can't put cursor beyond 4MB */
+            b3=(unsigned long *)p5;
+            switch(p2) {
+                case 0:
+                    cur_colors[p1*6]=(p3&0xff0000)>>16;
+                    cur_colors[p1*6+1]=(p3&0xff00)>>8;
+                    cur_colors[p1*6+2]=p3&0xff;
+                    cur_colors[p1*6+3]=(p4&0xff0000)>>16;
+                    cur_colors[p1*6+4]=(p4&0xff00)>>8;
+                    cur_colors[p1*6+5]=p4&0xff;
+                    for(j=0;j<32;j++) {
+                        l1=*(b3+j);
+                        l2=~(*(b3+32+j));
+                        for(k=0;k<8;k++) {
+                            c=0;
+                            for(l=0;l<4;l++) {
+                                c<<=1;
+                                c|=(l2>>31);
+                                c<<=1;
+                                c|=(l1>>31);
+                                if(c&2)c&=0xfe;
+                                l1<<=1;
+                                l2<<=1;
+                            }
+                            *(LINEAR_MEM_POINTER+i+16*j+k)=c;
+                            *(LINEAR_MEM_POINTER+i+16*j+k+8)=0xaa;
+                        }
+                    }
+                    memset(LINEAR_MEM_POINTER+i+512,0xaa,512);
+                    break;
+            }
+            break;
+    }
+    return 0;
+}       
+
 static int sis_match_programmable_clock(int clock)
 {
 return clock ;
@@ -501,6 +579,7 @@ DriverSpecs __svgalib_sis_driverspecs =
     sis_linear,
     0,				/* accelspecs, filled in during init. */
     NULL,                       /* Emulation */
+    sis_cursor
 };
 
 /* Initialize chipset (called after detection) */
@@ -570,6 +649,7 @@ static int sis_init(int force, int par1, int par2)
             sis_memory=1024<<(temp&0x03);
         };
     };
+
     if (__svgalib_driver_report) {
 	printf("Using SIS driver, %iKB. Chiptype=%i\n",sis_memory,sis_chiptype);
     };
