@@ -88,6 +88,8 @@ static int sk_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
+    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
+
     if ((mode < G640x480x256 )
 	|| mode == G720x348x2)
 	return __svgalib_vga_driverspecs.modeavailable(mode);
@@ -194,20 +196,10 @@ static void sk_lock(void)
 
 static int sk_test(void)
 {
-    int _ioperm;
-    
-    if (getenv("IOPERM") == NULL) {
-      _ioperm=1;
-      if (iopl(3) < 0) {
-        printf("svgalib: cannot get I/O permissions\n");
-        exit(1);
-      }
-    }
+    int found;
     
     found=__svgalib_pci_find_vendor_vga(VENDOR_ID,buf,0);
     
-    if (_ioperm) iopl(0);
-    
     if(!found&&((buf[0]>>16)&0xffff==CARD_ID)){
        sk_init(0,0,0);
        return 1;
@@ -296,7 +288,6 @@ static int sk_init(int force, int par1, int par2)
 {
     unsigned long buf[64];
     int found=0;
-    int _ioperm=0;
 
     sk_unlock();
     if (force) {
@@ -306,15 +297,7 @@ static int sk_init(int force, int par1, int par2)
 
     };
 
-    if (getenv("IOPERM") == NULL) {
-      _ioperm=1;
-      if (iopl(3) < 0) {
-        printf("svgalib: cannot get I/O permissions\n");
-        exit(1);
-      }
-    }
     found=__svgalib_pci_find_vendor_vga(VENDOR_ID,buf,0);
-    if (_ioperm) iopl(0);
     sk_linear_base=0;
     if (!found){
        sk_linear_base=buf[4]&0xffffff00;
