@@ -217,17 +217,20 @@ static void vesa_lock(void)
 
 static int vesa_test(void)
 {
-
         __svgalib_LRMI_init();
+        LRMI_mem2 = __svgalib_LRMI_alloc_real(sizeof(struct vbe_info_block)
+                                        + sizeof(struct vbe_mode_info_block));
 	vesa_data.info = LRMI_mem2;
 	vesa_data.mode = (struct vbe_mode_info_block *)(vesa_data.info + 1);
 	vesa_r.eax = 0x4f00;
 	vesa_r.es = (unsigned int)vesa_data.info >> 4;
 	vesa_r.edi = 0;
 
+        __svgalib_LRMI_free_real(LRMI_mem2);
+
         __svgalib_LRMI_int(0x10, &vesa_r);
-       if (vesa_r.eax!=0x4f) return 0;
-       return !vesa_init(0,0,0);
+        if (vesa_r.eax!=0x4f) return 0;
+        return !vesa_init(0,0,0);
 }
 
 
@@ -341,19 +344,18 @@ static int vesa_init(int force, int par1, int par2)
 {  
     short int *mode_list;
     int i;
+#if 0
     unsigned char *m;
     int address;
 
-#if 0
     m=mmap(0,0x502,PROT_READ,MAP_SHARED,__svgalib_mem_fd,0);
     
     address=((long)*(unsigned short *)(m+64))+(((long)*(unsigned short *)(m+66))<<4);
     if((address<0xa0000)||(address>0xfffff)) {
-       	fprintf(stderr, "Real mode int 0x10 handler not in ROM.\n");
+       	fprintf(stderr, "Real mode int 0x10 at 0x%08x handler not in ROM.\n",address);
         fprintf(stderr, "Try running vga_reset.\n");
        	return 1;
     };
-printf("%x\n",address    );
 #endif
     __svgalib_textprog|=1;
 
