@@ -16,9 +16,6 @@
 int vga_clear(void)
 {
     vga_screenoff();
-#ifdef BACKGROUND
-    __svgalib_dont_switch_vt_yet();
-#endif
     if (MODEX)
 	goto modeX;
     switch (CM) {
@@ -27,29 +24,12 @@ int vga_clear(void)
     case G320x400x256:
     case G360x480x256:
       modeX:
-#ifdef BACKGROUND
-        if (vga_oktowrite())
-	    {
-#endif
         
 	/* write to all planes */
         __svgalib_outseq(0x02,0x0f);
         
 	/* clear video memory */
 	memset(GM, 0, 65536);
-#ifdef BACKGROUND
-	    }
-	  else
-	    {
-	     int i;
-	     
-	     for (i = 0; i < 4; i++) {
-	        /* save plane i */
-	        __svgalib_fast_setpage(i);
-	        memset(GM, 0, GRAPH_SIZE);
-               }
-	    }
-#endif
 	break;
 
     default:
@@ -58,15 +38,8 @@ int vga_clear(void)
 	case 16:
 	    vga_setcolor(0);
 
-#ifdef BACKGROUND
-        if (vga_oktowrite())
-	    {
-#endif
 	    /* write to all bits */
             __svgalib_outseq(0x08,0xff);
-#ifdef BACKGROUND
-	    }
-#endif
 
 	default:
 	    {
@@ -75,17 +48,6 @@ int vga_clear(void)
 #if defined(CONFIG_ALPHA_JENSEN)
 		int j;
 #endif
-#ifdef BACKGROUND
-                if (!vga_oktowrite())
-	              {
-		       switch (CI.colors) {
-	                case 2:
-	                case 16:
-			pages*=4;
-                       }
-		      }
-#endif
-
 		for (i = 0; i < pages; ++i) {
 		    vga_setpage(i);
 
@@ -104,9 +66,6 @@ int vga_clear(void)
     }
 
     vga_setcolor(15);
-#ifdef BACKGROUND
-    __svgalib_is_vt_switching_needed();
-#endif
     vga_screenon();
 
     return 0;
