@@ -12,10 +12,6 @@
 #include <stdio.h>
 #include "vga.h"
 #include "libvga.h"
-#ifdef BACKGROUND
-#include "vgabg.h"
-#endif
-
 
 /* used to decompose color value into bits (for fast scanline drawing) */
 union bits {
@@ -161,46 +157,11 @@ void a_memcpy(unsigned char *dest, unsigned char *src, long n)
 
 #endif
 
-#ifdef BACKGROUND
-
-static int _vga_drawscansegment_bg(unsigned char *colors, 
-    int x, int y, int length)
-{
- /* The easy way */
- int count=0;
- int color;
- 
- color=COL;
- while(count<length)
-     {
-      vga_setcolor(*colors);
-      __svgalib_fast_drawpixel(x+count,y);
-      colors++;
-      count++;
-     }
- vga_setcolor(color);
- return(0);
-}
-#endif
-
-
 int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
 {
     /* both length and x must divide with 8 */
     /*  no longer true (at least for 16 & 256 colors) */
 
-#ifdef BACKGROUND
- __svgalib_dont_switch_vt_yet();
- 
- if (!vga_oktowrite())
-     {
-      /*__svgalib_is_vt_switching_needed(); */
-      _vga_drawscansegment_bg(colors,x,y,length);
-      __svgalib_is_vt_switching_needed();
-      return(0);
-     }
-#endif
-
     if (MODEX)
 	goto modeX;
     switch (CI.colors) {
@@ -378,9 +339,6 @@ int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
 		MEMCPY(SM + ((y * CI.xdim + x) << 7), colors, length);
 #else
 		MEMCPY(GM + (y * CI.xdim + x), colors, length);
-#endif
-#ifdef BACKGROUND
-                __svgalib_is_vt_switching_needed();
 #endif
 		return 0;
 	    case G320x240x256:
@@ -404,9 +362,6 @@ int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
 			}
 		    }
 		}
-#ifdef BACKGROUND
-                __svgalib_is_vt_switching_needed();
-#endif
 		return 0;
 	    }
 	    {
@@ -524,44 +479,11 @@ int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
 	}
     }
 
-#ifdef BACKGROUND
- __svgalib_is_vt_switching_needed();
-#endif
     return 0;
 }
 
-#ifdef BACKGROUND
-
-static int _vga_getscansegment_bg(unsigned char *colors, 
-    int x, int y, int length)
-{
- /* The easy way */
- int count=0;
- 
- while(count<length)
-     {
-      vga_setcolor(*colors);
-      *colors=__svgalib_fast_getpixel(x+count,y);
-      colors++;
-      count++;
-     }
- return(0);
-}
-#endif
-
 int vga_getscansegment(unsigned char *colors, int x, int y, int length)
 {
-#ifdef BACKGROUND
- __svgalib_dont_switch_vt_yet();
- 
- if (!vga_oktowrite())
-     {
-      /*__svgalib_is_vt_switching_needed(); */
-      _vga_getscansegment_bg(colors,x,y,length);
-      __svgalib_is_vt_switching_needed();
-      return(0);
-     }
-#endif
 
     if (MODEX)
 	goto modeX2;
@@ -649,9 +571,6 @@ int vga_getscansegment(unsigned char *colors, int x, int y, int length)
 	    switch (CM) {
 	    case G320x200x256:	/* linear addressing - easy and fast */
 		memcpy(colors, GM + y * CI.xdim + x, length);
-#ifdef BACKGROUND
-                __svgalib_is_vt_switching_needed();
-#endif
 		return 0;
 	    case G320x240x256:
 	    case G320x400x256:
@@ -672,9 +591,6 @@ int vga_getscansegment(unsigned char *colors, int x, int y, int length)
 			}
 		    }
 		}
-#ifdef BACKGROUND
-                __svgalib_is_vt_switching_needed();
-#endif
 		return 0;
 	    }
 	    {
@@ -702,13 +618,12 @@ int vga_getscansegment(unsigned char *colors, int x, int y, int length)
 	x *= 2;
 	goto SegmentedCopy2;
     case 1 << 24:
-#ifdef BACKGROUND
-        __svgalib_is_vt_switching_needed();
-#endif
-	return -1;		/* not supported */
+	if (__svgalib_cur_info.bytesperpixel == 4) {
+		x<<=2;
+	} else {
+		x *= 3;
+	}
+	goto SegmentedCopy2;
     }
-#ifdef BACKGROUND
-    __svgalib_is_vt_switching_needed();
-#endif
     return 0;
 }
