@@ -67,35 +67,8 @@ static int get_lut(int index, int *red, int *green, int *blue)
     return 0;
 }
 
-#ifdef BACKGROUND
-
-static void vga_setpalette_bg(int index, int red, int green, int blue)
-
-{
- unsigned char *tmp;
- /* asume max 256 palette places. */
- if (index<256)
-     {
-      tmp=__svgalib_give_graph_red();
-      tmp[index]=red;
-      tmp=__svgalib_give_graph_blue();
-      tmp[index]=blue;
-      tmp=__svgalib_give_graph_green();
-      tmp[index]=green;
-     } 
- return;
-}
-
-#endif
-
-
 int vga_setpalette(int index, int red, int green, int blue)
 {
-#ifdef BACKGROUND
- int tmp;
- 
- __svgalib_dont_switch_vt_yet();
-#endif
     if (__svgalib_grayscale) {
 	if ((unsigned) index >= sizeof(__svgalib_green_backup) / sizeof(__svgalib_green_backup[0])) {
 	    printf("vga_setpalette: color index %d out of range\n", index);
@@ -109,78 +82,24 @@ int vga_setpalette(int index, int red, int green, int blue)
 	    green = 255;
     }
 
-#ifdef BACKGROUND
-    if (!vga_oktowrite()) {
-	vga_setpalette_bg(index, red, green, blue);
-	__svgalib_is_vt_switching_needed();
-	return(0);
-    }
-    if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->setpalette) {
-        tmp =__svgalib_driverspecs->emul->setpalette(index, red, green, blue);
-    } else {
-	tmp = set_lut(index, red, green, blue);
-    }
-    __svgalib_is_vt_switching_needed();
-    return(tmp);
-#else
     if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->setpalette) {
         return __svgalib_driverspecs->emul->setpalette(index, red, green, blue);
     } else {
 	return set_lut(index, red, green, blue);
     }
-#endif
 }
 
-#ifdef BACKGROUND
-
-static void vga_getpalette_bg(int index, int *red, int *green, int *blue)
-{
- unsigned char *tmp;
- /* asume max 256 palette places. */
- if (index<256)
-     {
-      tmp=__svgalib_give_graph_red();
-      *red=tmp[index];
-      tmp=__svgalib_give_graph_blue();
-      *blue=tmp[index];
-      tmp=__svgalib_give_graph_green();
-      *green=tmp[index];
-     } 
- return;
-}
-
-#endif
-
 int vga_getpalette(int index, int *red, int *green, int *blue)
 {
-#ifdef BACKGROUND
- __svgalib_dont_switch_vt_yet();
-#endif
-#ifndef BACKGROUND
     if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->getpalette) 
         __svgalib_driverspecs->emul->getpalette(index, red, green, blue);
     else get_lut(index, red, green, blue);
-#endif
-#ifdef BACKGROUND
- if (!vga_oktowrite())
-     {
-      vga_getpalette_bg(index, red, green, blue);
-     }
-   else {
-    if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->getpalette) 
-        __svgalib_driverspecs->emul->getpalette(index, red, green, blue);
-    else get_lut(index, red, green, blue);
-   }
-#endif
     if (__svgalib_grayscale) {
 	if ((unsigned) index >= sizeof(__svgalib_green_backup) / sizeof(__svgalib_green_backup[0])) {
 	    printf("vga_getpalette: color index %d out of range\n", index);
 	}
 	*green = __svgalib_green_backup[index];
     }
-#ifdef BACKGROUND
- __svgalib_is_vt_switching_needed();
-#endif
     return 0;
 }
 
@@ -189,10 +108,24 @@ int vga_setpalvec(int start, int num, int *pal)
 {
     int i;
 
-    for (i = start; i < start + num; ++i) {
-	vga_setpalette(i, pal[0], pal[1], pal[2]);
-	pal += 3;
+    if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->setpalette) {
+        for (i = start; i < start + num; ++i) {
+	    vga_setpalette(i, pal[0], pal[1], pal[2]);
+	    pal += 3;
+        }
+    } else {
+        unsigned char string[768];
+
+        if ( num > 256 )
+            return 0;
+
+        for (i = 0; i < num * 3; i++)
+            string[i] = pal[i];
+
+        port_out ( start, 0x3c8 );
+        port_rep_outb( string, num * 3, 0x3c9 );
     }
+
     return num;
 }
 
